#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <string.h>

typedef struct routes
{
  uint32_t ip;
  uint32_t sm;
} route;

/* Create a byte array containing the octets of an IP address from
   an integer representation of that IP. The results are stored
   in the provided unsigned character array. The length of this
   array must be four, and the length argument must also be four.
   C does not, to my knowledge, contain a mechanism to test an
   assertion that the buffer really is big enough. */
unsigned char * iptouca
(const uint32_t ip, unsigned char * dst, const int length);

/* Convert an IP from integer value to dotted-decimal string representation.
   If dst is null, a new bit-byte buffer will be allocated for the string,
   but it is still the responsibility of the caller to free() this string. */
char * ipntoa
(const uint32_t ip, char * dst, const int length);

/* Given an IP address and its subnet mask,
   calculate the directed subnet broadcast. */
uint32_t subnetDirectedBroadcast
(route r);

/* A wildcard mask is simply a subnet mask with inverted bits. */
uint32_t wildcardMask
(uint32_t sm);

/* Finds what class a given IP belongs to.
   See RFC 791 and 1112. */
char classful
(const uint32_t ip);

/* Find the classful netmask for a given IP.
   Behavior is not defined for Class D and E. */
uint32_t classfulMask
(const uint32_t ip);

/* Using the Find First Set function, determine
   the prefix length for an IPv4 subnet mask. */
uint32_t prefixLength
(const uint32_t sm);

/* Find the number of hosts in a given subnet.
   0.0.0.0 and 255.255.255.255 return 0.
   255.255.255.254 returns 2. */
uint32_t countHosts
(const uint32_t sm);

/* Identifies whether a subnet mask is valid using popcnt instruction.
   Compile with -mpopcnt, -msse4.2, or -march=corei7 for best performance.
   http://www.dalkescientific.com/writings/diary/archive/2011/11/02/faster_popcount_update.html
   Check with objdump -d a.out | grep popcnt
   Also interested in POSIX FFS vs. GNU CC builtin FFS.

   Returns 1 for valid netmask, 0 otherwise. Prints warning to
   stderr if the netmask is 0.0.0.0 or 255.255.255.255. */
int isValidNetmask
(const uint32_t sm);

/* A simple stab at parsing IP addresses.
   It isn't mean to be perfect. */
uint32_t parse
(const char * src);

/* Generates all possible 32-bit subnet masks. */
void generateSubnets
();

/* Given an IP address and its subnet mask,
   calculate the network number.
   See RFC 950. */
uint32_t networkNumber
(uint32_t ip, uint32_t sm);

/* Calculates the number of subnets smaller than classful boundry.
   If subnet mask prefix length is shorter than that of the IP's
   classful prefix, such as 10.0.0.0/7, the function returns 0. */
uint32_t countSubnets
(const uint32_t ip, const uint32_t sm);

/* Given two IP address and corresponding subnet masks,
   calculate a summary address that includes each subnet. */
uint32_t summaryAddress
(uint32_t ip1, uint32_t sm1, uint32_t ip2, uint32_t sm2);

/* Given two IP address and corresponding subnet masks,
   calculate the subnet mask for a summary route.
   TODO: improve algorithm! */
uint32_t summaryPrefix
(uint32_t ip1, uint32_t sm1, uint32_t ip2, uint32_t sm2);

/* Generates routes along classful boundary for given
   summary. If the summary has a prefix shorter than
   that of the classful network, then more than one
   classful route will be pushed to returned route array.
   Find the length of this array by calling summaryClassfulCount.
   The caller MUST free() the output of this function
   to avoid memory leaks. */
route * summaryClassful
(route r1);

/* Finds the number of classful networks associated with
   a summary route. */
int summaryClassfulCount
(route r1);
