#include "ipv4.h"

#define IP_STRLEN 16

#ifdef COLORS
#define ANSI_COLOR_RED     "\x1b[31m"
#define ANSI_COLOR_GREEN   "\x1b[32m"
#define ANSI_COLOR_YELLOW  "\x1b[33m"
#define ANSI_COLOR_BLUE    "\x1b[34m"
#define ANSI_COLOR_MAGENTA "\x1b[35m"
#define ANSI_COLOR_CYAN    "\x1b[36m"
#define ANSI_COLOR_RESET   "\x1b[0m"
#else
#define ANSI_COLOR_RED     ""
#define ANSI_COLOR_GREEN   ""
#define ANSI_COLOR_YELLOW  ""
#define ANSI_COLOR_BLUE    ""
#define ANSI_COLOR_MAGENTA ""
#define ANSI_COLOR_CYAN    ""
#define ANSI_COLOR_RESET   ""
#endif

const char * usage = "Usage: ./ipv4 [ip1 sm1] {ip2 sm2} | [generate subnets]";

void testNetworkNumber
(const char * ip1, const char * sm1,
 const char * ip2, const char * sm2)
{
    uint32_t ip1_i, ip2_i, sm1_i, sm2_i;
    char network1[IP_STRLEN], network2[IP_STRLEN];

    ip1_i = parse(ip1);
    sm1_i = parse(sm1);
    if (isValidNetmask(sm1_i) == 0)
      {
	exit(-1);
      }
    ipntoa(networkNumber(ip1_i, sm1_i), network1, IP_STRLEN);

    if (ip2 != NULL && sm2 != NULL)
      {
	ip2_i = parse(ip2);
	sm2_i = parse(sm2);
	if (isValidNetmask(sm2_i) == 0)
	  {
	    exit(-1);
	  }
	ipntoa(networkNumber(ip2_i, sm2_i), network2, IP_STRLEN);
      }

    printf("                NETWORK NUMBERS\n");
    printf("IP                Subnet Mask       Prefix  Network Number   Class\n");
    printf("%-15s   %-15s   %-2d      %-15s  %c\n", ip1, sm1, prefixLength(sm1_i), network1, classful(ip1_i));
    if (ip2 != NULL && sm2 != NULL)
      printf("%-15s   %-15s   %-2d      %-15s  %c\n", ip2, sm2, prefixLength(sm2_i), network2, classful(ip2_i));
}

void testSubnet
(const char * ip1, const char * sm1,
 const char * ip2, const char * sm2)
{
  route r1, r2;
  uint32_t ip1_i, ip2_i, sm1_i, sm2_i, bc1, bc2;
  char broadcast1[IP_STRLEN];
  char broadcast2[IP_STRLEN];

  ip1_i = parse(ip1);
  sm1_i = parse(sm1);
  r1.ip = ip1_i;
  r1.sm = sm1_i;
  bc1 = subnetDirectedBroadcast(r1);
  ipntoa(bc1, broadcast1, IP_STRLEN);

  if (ip2 != NULL && sm2 != NULL)
    {
      ip2_i = parse(ip2);
      sm2_i = parse(sm2);
      r2.ip = ip2_i;
      r2.sm = sm2_i;
      bc2 = subnetDirectedBroadcast(r2);
      ipntoa(bc2, broadcast2, IP_STRLEN);
    }

  printf("             SUBNET DIRECTED BROADCAST\n");
  printf("IP                Subnet Mask       Prefix  Broadcast\n");
  printf("%-15s   %-15s   %-2d      %-15s\n", ip1, sm1, prefixLength(sm1_i), broadcast1);
  if (ip2 != NULL && sm2 != NULL)
    printf("%-15s   %-15s   %-2d      %-15s\n", ip2, sm2, prefixLength(sm2_i), broadcast2);
}

void testNetworkSize
(const char * ip1, const char * sm1,
 const char * ip2, const char * sm2)
{
  uint32_t ip1_i, ip2_i, sm1_i, sm2_i, net1_i, net2_i;
  char cidr1[20], cidr2[20], net1_s[IP_STRLEN], net2_s[IP_STRLEN];

  ip1_i = parse(ip1);
  sm1_i = parse(sm1);
  net1_i = networkNumber(ip1_i, sm1_i);
  ipntoa(net1_i, net1_s, IP_STRLEN);
  snprintf(cidr1, 20, "%s/%d", net1_s, prefixLength(sm1_i));

  if (ip2 != NULL && sm2 != NULL)
    {
      ip2_i = parse(ip2);
      sm2_i = parse(sm2);
      net2_i = networkNumber(ip2_i, sm2_i);
      ipntoa(net2_i, net2_s, IP_STRLEN);
      snprintf(cidr2, 20, "%s/%d", net2_s, prefixLength(sm2_i));
    }

  printf("                  NETWORK SIZE\n");
  printf("Network             Subnets\tHosts\n");
  printf("%-18s  %-7d\t%d\n", cidr1, countSubnets(ip1_i, sm1_i), countHosts(sm1_i));
  if (ip2 != NULL && sm2 != NULL)
    printf("%-18s  %-7d\t%d\n", cidr2, countSubnets(ip2_i, sm2_i), countHosts(sm2_i));
}

void testClassfulRoutes
(route r1)
{
  int k, r;
  route * classfulRoutes;

  r = summaryClassfulCount(r1);
  classfulRoutes = summaryClassful(r1);

  fprintf(stderr, "Number of routes: %d\n", r);
  for (k=0; k<r; k++)
    {
      char crs[IP_STRLEN];
      fprintf(stderr, "\t%s/%d\n",
	      ipntoa(classfulRoutes[k].ip, crs, IP_STRLEN),
	      prefixLength(classfulRoutes[k].sm));
    }
  free(classfulRoutes);
}

void testSummary
(const char * ip1, const char * sm1,
 const char * ip2, const char * sm2)
{
  uint32_t ip1_i, ip2_i, sm1_i, sm2_i, sumroute, sumpfx;
  route r1;

  /* Summary route network ID */
  char sip[IP_STRLEN];
  /* Summary route subnet mask */
  char ssm[IP_STRLEN];
  /* Summary route wildcard mask */
  char wcm[IP_STRLEN];

  ip1_i = parse(ip1);
  ip2_i = parse(ip2);
  sm1_i = parse(sm1);
  sm2_i = parse(sm2);

  sumroute = summaryAddress(ip1_i, sm1_i, ip2_i, sm2_i);
  sumpfx = summaryPrefix(ip1_i, sm1_i, ip2_i, sm2_i);

  ipntoa(sumroute, sip, IP_STRLEN);
  ipntoa(sumpfx, ssm, IP_STRLEN);
  ipntoa(wildcardMask(sumpfx), wcm, IP_STRLEN);

  printf("                 SUMMARY ROUTE\n");
  printf("Summary IP        Summary Mask      Prefix   Wildcard Mask\n");
  printf("%-15s   %-15s   %-2d       %-15s\n", sip, ssm, prefixLength(sumpfx), wcm);

  printf("\n                CLASSFUL SUMMARY\n");
  r1.ip = sumroute;
  r1.sm = sumpfx;
  testClassfulRoutes(r1);
}

void testAll
(const char * ip1, const char * sm1,
 const char * ip2, const char * sm2)
{
  testNetworkNumber(ip1, sm1, ip2, sm2);
  printf("\n");
  testSubnet(ip1, sm1, ip2, sm2);
  printf("\n");
  testNetworkSize(ip1, sm1, ip2, sm2);
  printf("\n");
  testSummary(ip1, sm1, ip2, sm2);
}

int main
(int argc, char ** argv)
{
  /* If no arguments provided, give help and quit. */
  int i;
  for (i = 1; i < argc; i++)
    {
      if (strcmp(argv[i], "-h") == 0 || strcmp(argv[i], "--help") == 0)
	{
	  fprintf(stderr, ANSI_COLOR_YELLOW "%s" ANSI_COLOR_RESET, usage);
	  return(EXIT_SUCCESS);
	}
    }

  /* Five arguments: ./test, IP1, SM1, IP2, SM2 */
  if (argc == 5)
    {
      testAll(argv[1], argv[2], argv[3], argv[4]);
    }
  /* Generate subnets makes a list of possible masks. */
  else if (argc== 3 && (strncmp(argv[1], "generate", 10) == 0) && 
	   (strncmp(argv[2], "subnets", 8) == 0))
    {
      generateSubnets();
    }
  /* Three arguments: ./test, IP, SM */
  else if (argc == 3)
    {
      testNetworkNumber(argv[1], argv[2], NULL, NULL);
      printf("\n");
      testSubnet(argv[1], argv[2], NULL, NULL);
      printf("\n");
      testNetworkSize(argv[1], argv[2], NULL, NULL);
    }
  else
    {
      fprintf(stderr, ANSI_COLOR_YELLOW "%s\n" ANSI_COLOR_RESET, usage);
    }

  return(EXIT_SUCCESS);
}
